#include <stdlib.h>
#include <stdio.h>
#include "search.h"
#include "proc_ora.h"

#define CL_TOP 0
#define CL_WORD 1
#define CL_AND 2
#define CL_OR 3
#define CL_NOT 4
#define CL_FM 5
#define CL_SP 6
#define CL_PRX 7
#define CL_PHR 8
#define CL_T 9
#define CL_SYN 10


/*typedef struct string
{
  int   block_len;
  int   str_len;
  char* string;
} string;
*/  

typedef struct element
{
  int		  element_id;
  int		  inf;
  struct string	  txt;
  struct string	  book_name;
  struct string   sqe_name;
  struct element *children;
  struct element *next;
} element;


void str_append(string*,char*);
void str_prepend(string*,char*);
void str_cpy(string*,char*);
void str_cat(string*,string*);
void str_dup(string*,string*);
void str_free(string*);
int  str_len(string*);
int  get_next_word(string*,string*,int);



static element top={CL_TOP,0,{0,0,0},0,0};
const element zero_element={CL_WORD,0,{0,0,0},0,0};

void new_class_and(element* elem,string *txt, int info);
void new_class_or(element* elem,string *txt, int info);
void new_class_not(element* elem,string *txt, int info);
void new_class_t(element* elem,string *txt, int info);
void new_class_fm(element* elem,string *txt, int info);
void new_class_sp(element* elem,string *txt, int info);
void new_class_syn(element* elem,string *txt, int info);
void new_class_prx(element* elem,string *txt, int info);
void new_class_phr(element* elem,string *txt, int info);
void new_class_not(element* elem,string *txt, int info);
void class_build(element* cur_elem,int rel_type,int info);

element* new_element()
{
  element * el=(element*)malloc(sizeof(element));
  memcpy(el,&zero_element,sizeof(element));
  return el;
}

void create_top(char * str, char* book, char* sqe)
{
  top.element_id=CL_TOP;
  str_cpy(&top.txt,"");
  str_cpy(&top.book_name,book);
  str_cpy(&top.sqe_name,sqe);
  top.children=new_element();
  top.children->element_id=CL_WORD;
  str_cpy(&(top.children->txt),str);
  top.next=0;
}

void object_build(element* cur_elem,int rel_type,int info)
{
  element *tmp,*point;
  for(point=cur_elem->children; point!=NULL; point=point->next)
    class_build(point,rel_type,info);
}

void class_build(element* cur_elem,int rel_type,int info)
{
  switch(cur_elem->element_id)
  {
  case CL_TOP:
    object_build(cur_elem,rel_type,info);
    break;
  case CL_WORD:
    switch(rel_type)
    {
    case NOT_EXPR:  new_class_not(cur_elem,&cur_elem->txt,info); break;
    case AND_EXPR:  new_class_and(cur_elem,&cur_elem->txt,info); break;
    case OR_EXPR:   new_class_or(cur_elem,&cur_elem->txt,info);  break;
    case T_EXPR:    new_class_t(cur_elem,&cur_elem->txt,info);   break;
    case FM_EXPR:   new_class_fm(cur_elem,&cur_elem->txt,info);  break;
    case SP_EXPR:   new_class_sp(cur_elem,&cur_elem->txt,info);  break;
    case SYN_EXPR:  new_class_syn(cur_elem,&cur_elem->txt,info); break; 
    case PRX_EXPR:  new_class_prx(cur_elem,&cur_elem->txt,info); break; 
    case PHR_EXPR:  new_class_phr(cur_elem,&cur_elem->txt,info); break; 
    case RM_NONEX:  if(not_exist(cur_elem->txt.string)) str_cpy(&cur_elem->txt,"");
    }
    break;
  case CL_FM:
    if(rel_type!=RM_NONEX) object_build(cur_elem,rel_type,info);
    break;
  case CL_SP:
    if(rel_type!=RM_NONEX) object_build(cur_elem,rel_type,info);
    break;
  case CL_PRX:
    break;
  case CL_PHR:
    break;
  default :
    object_build(cur_elem,rel_type,info);
  }
}

void build(int rel_type,int info)
{
  class_build(&top,rel_type,info);
}


object_print(element *cur_elem,string *str)
{
  string  tmp_str1={0,0,0};
  element *elem_point;
  int i;
  switch(cur_elem->element_id)
  {
  case CL_TOP:
    object_print(cur_elem->children,&tmp_str1);
    str_cpy(str,"SELECT doc FROM search WHERE ");
    if(strcmp(cur_elem->book_name.string,NULL)!=0)
    {
      /*str_append(str,"part_of='");*/
      str_cat(str,&cur_elem->book_name);
      str_append(str," AND ");
    }
    str_append(str,"doc CONTAINS ");
    str_cat(str,&tmp_str1);
    if(str_len(&cur_elem->sqe_name)!=0)
    {
      str_append(str," AND SQE(");
      str_cat(str,&cur_elem->sqe_name);
      str_append(str,")");
    }
    break;
  case CL_NOT:
    object_print(cur_elem->children,str);
    str_prepend(str,"NOT(");
    str_append(str,")");
    break;
  case CL_AND:
    str_cpy(str,"");
    i=0;
    for(elem_point=cur_elem->children; elem_point!=NULL; elem_point=elem_point->next)
    {
      string tmp_str2={0,0,0};
      object_print(elem_point,&tmp_str2);
      if(str_len(&tmp_str2)!=0)
      {
	i++;
	if(str_len(str)==0)
	  str_cat(str,&tmp_str2);
	else
	{
	  str_append(str," AND ");
	  str_cat(str,&tmp_str2);
	}
      }
      str_free(&tmp_str2);
    }
    if(i>1)
    {
      str_prepend(str,"(");
      str_append(str,")");
    }
    break;
  case CL_OR:
    str_cpy(str,"");
    i=0;
    for(elem_point=cur_elem->children; elem_point!=NULL; elem_point=elem_point->next)
    {
      string tmp_str2={0,0,0};
      object_print(elem_point,&tmp_str2);
      if(str_len(&tmp_str2)!=0)
      {
	i++;
	if(str_len(str)==0)
	  str_cat(str,&tmp_str2);
	else
	{
	  str_append(str," OR ");
	  str_cat(str,&tmp_str2);
	}
      }
      str_free(&tmp_str2);
    }
    if(i>1)
    {
      str_prepend(str,"(");
      str_append(str,")");
    }
    break;
  case CL_T:
    if(cur_elem->inf & CASE_SENS)
    {
      object_print(cur_elem->children,str);
      if(cur_elem->inf & FT) str_prepend(str,"*");
      if(cur_elem->inf & BT) str_append(str,"*");
    }
    else
    {
      object_print(cur_elem->children,str);
    }
    break;
  case CL_FM:
    object_print(cur_elem->children,str);
    str_prepend(str,"FM(");
    str_append(str,")");
    break;
  case CL_SP:
    object_print(cur_elem->children,str);
    str_prepend(str,"SP(");
    str_append(str,")");
    break;
  case CL_SYN:
    object_print(cur_elem->children,str);
    break;
  case CL_PRX:
  {
    char operator[30];
    sprintf(operator,"(%d,%d)",cur_elem->inf % 128,(int)(cur_elem->inf/128));
    str_cpy(str,"");
    i=0;
    for(elem_point=cur_elem->children; elem_point!=NULL; elem_point=elem_point->next)
    {
      string tmp_str2={0,0,0};
      object_print(elem_point,&tmp_str2);
      if(str_len(&tmp_str2)!=0)
      {
	if(str_len(str)==0)
	  str_cat(str,&tmp_str2);
	else
	{
	  str_append(str,operator);
	  str_cat(str,&tmp_str2);
	  i++;
	}
      }
      str_free(&tmp_str2);
    }
    if(i>1)
    {
      str_prepend(str,"(");
      str_append(str,")");
    }
  }
    break;
    
  default :
    str_cpy(str,"");
    if(str_len(&cur_elem->txt)>0)
    {
      str_cat(str,&cur_elem->txt);
      str_prepend(str,"'");
      str_append(str,"'");
    }
    break;
  }
  str_free(&tmp_str1);
}

void remove_object(element* elem)
{
  if(elem->next!=NULL) {remove_object(elem->next); free(elem->next);}
  if(elem->children!=NULL) {remove_object(elem->children);free(elem->children);}
  str_free(&elem->txt);
}
  

void remove_top()
{
  remove_object(&top);
}

void new_class_and(element* elem,string *txt, int info)
{
  int position=0;
  string str={0,0,0};
  element ** previous = &(elem->children);
  elem->element_id=CL_AND; 
  while(position=get_next_word(txt,&str,position))
  {
    element * el=new_element();
    str_dup(&el->txt,&str);
    el->element_id=CL_WORD;
    el->children=0;
    el->next=0;
    *previous=el;
    previous=&el->next;
  }
  str_free(&elem->txt);
  str_free(&str);
}

void new_class_or(element* elem,string *txt, int info)
{
  int position=0;
  string str={0,0,0};
  element ** previous = &(elem->children);
  elem->element_id=CL_OR; 
  while(position=get_next_word(txt,&str,position))
  {
    element * el=new_element();
    str_dup(&el->txt,&str);
    el->element_id=CL_WORD;
    el->children=0;
    el->next=0;
    *previous=el;
    previous=&el->next;
  }
  str_free(&elem->txt);
  str_free(&str);
}



void new_class_t(element* elem,string *txt, int inf)
{
  string old_str={0,0,0};
  element *el=new_element();

  elem->inf=inf;
  str_dup(&el->txt,txt);
  elem->element_id=CL_T;
  elem->children=el;
  el->element_id=CL_WORD;
  el->children=0;
  el->next=0;
  if(!(inf & CASE_SENS))
  {
    if(inf & FT) str_append(&el->txt,"%");
    if(inf & BT) str_prepend(&el->txt,"%");
    str_cpy(&el->txt,find_matching_words(el->txt.string));
    if(inf & PR_FLAG)
      printf("\nStem expansion of %s: %s\n",txt->string,el->txt.string); 
   str_prepend(txt," ");
    str_append(txt," ");
    str_append(&el->txt," ");
    if(!strstr(el->txt.string,txt->string))
      str_cat(&el->txt,txt);
    new_class_or(el,&el->txt,inf);
  }
  str_free(&old_str);
}



void new_class_fm(element* elem,string *txt, int info)
{
  element *el;
  
  el=new_element();
  str_dup(&el->txt,&elem->txt);
  elem->children=el;
  elem->element_id=CL_FM;
}
  
void new_class_sp(element* elem,string *txt, int info)
{
  element *el;
  
  el=new_element();
  memcpy(el,&zero_element,sizeof(element));
  str_dup(&el->txt,&elem->txt);
  elem->children=el;
  elem->element_id=CL_SP;
  str_free(&elem->txt);
}
    
void new_class_syn(element* elem,string *txt, int info)
{
  string str={0,0,0};
  element *el;
  str_cpy(&str,find_synonyms(txt->string));
  if(!strstr(str.string,txt->string))
  {
    str_append(&str," ");
    str_cat(&str,txt);
  }
/*  printf("\nSynonym expansion of %s: %s\n",txt->string,str.string);*/
  el=new_element();
  memcpy(el,&zero_element,sizeof(element));
  memcpy(&el->txt,&str,sizeof(string));
  elem->children=el;
  elem->element_id=CL_SYN;
  new_class_or(el,&str,info);
}

void new_class_prx(element* elem,string *txt, int info)
{
  int position=0;
  string str={0,0,0};
  element ** previous=&elem->children;
  elem->inf=info;
  elem->element_id=CL_PRX;
  while(position=get_next_word(txt,&str,position))
  {
    element * el=new_element();
    str_dup(&el->txt,&str);
    el->element_id=CL_WORD;
    el->children=0;
    el->next=0;
    *previous=el;
    previous=&el->next;
  }
  str_free(&elem->txt);
  str_free(&str);
}
  
void new_class_phr( element* elem,string *txt, int info)
{
  element *el;
  
  el=new_element();
  memcpy(el,&zero_element,sizeof(element));
  str_dup(&el->txt,&elem->txt);
  elem->children=el;
  elem->element_id=CL_PHR;
}

void new_class_not(element* elem,string *txt, int info)
{
  element *el;
  
  el=new_element();
  memcpy(el,&zero_element,sizeof(element));
  str_dup(&el->txt,&elem->txt);
  elem->children=el;
  elem->element_id=CL_NOT;
}


int str_len(string* str){return str->str_len;}
void str_cpy(string* str,char* txt)
{
  if(str->block_len==0)
  {
    str->block_len=strlen(txt)+16;
    str->string=(char*)malloc(str->block_len);
    strcpy(str->string,txt);
    str->str_len=strlen(txt);
  }
  else
  {
    str->block_len=strlen(txt)+16;
    str->string=(char*)realloc(str->string,str->block_len);
    strcpy(str->string,txt);
    str->str_len=strlen(txt);
  }
  str->string[str->str_len]='\0';
}
void str_append(string* str,char* txt)
{
  if(str->block_len==0)
  {
    str->block_len=strlen(txt)+16;
    str->string=(char*)malloc(str->block_len);
    strcpy(str->string,txt);
    str->str_len=strlen(txt);
  }
  else
  {
    if(str->block_len<=str->str_len+strlen(txt))
    {
      str->block_len=str->str_len+strlen(txt)+16;
      str->string=(char*)realloc(str->string,str->block_len);
      strcpy(str->string+str->str_len,txt);
      str->str_len+=strlen(txt);
    }
    else
    {
      strcpy(str->string+str->str_len,txt);
      str->str_len+=strlen(txt);
    }
  }
  str->string[str->str_len]='\0';
}
void str_prepend(string* str,char* txt)
{
  if(str->block_len==0)
  {
    str->block_len=strlen(txt)+16;
    str->string=(char*)malloc(str->block_len);
    strcpy(str->string,txt);
    str->str_len=strlen(txt);
  }
  else
  {
      char* new_str;
      str->str_len+=strlen(txt);
      str->block_len=str->str_len+16;
      new_str=(char*)malloc(str->block_len);
      strcpy(new_str,txt);
      strcat(new_str,str->string);
      free(str->string);
      str->string=new_str;
  }
}
void str_cat(string* a, string* b)
{
  if(b->string!=NULL) str_append(a,b->string);
}
void str_dup(string* a,string* b)
{
  if(b->string!=NULL) str_cpy(a,b->string);
}
void str_free(string* str)
{
  if(str->string!=NULL)
  {
    free(str->string);
    str->block_len=0;
    str->str_len=0;
  }
}
int get_next_word(string *str,string *substr,int pos)
{
  char* p1=NULL;
  int i,n=0;
  for(i=pos; i<str->str_len ; i++)
    if(str->string[i]>' ') break;
  if(i==str->str_len) return 0;
  if(i<str->str_len) p1=str->string+i;
  while(i+n<str->str_len && str->string[i+n]>' ') n++;
  str_free(substr);
  substr->block_len=n+16;
  substr->str_len=n;
  substr->string=(char*)malloc(substr->block_len);
  memcpy(substr->string,str->string+i,n);
  substr->string[n]='\0';
  return n+i;
}

char *print()
{
  static string str={0,0,0};
  str_free(&str);
  object_print(&top,&str);
/*printf("QUERY: %s<br>\n",str.string); */
  return str.string;
}

void output_info(global_params *glp)
{
  extern FILE* info_file;
  char buffer[512];
  string info={0,0,0};
  if(info_file==NULL) return;
  rewind(info_file);
  while(fgets(buffer,512,info_file)!=NULL)
  {
    int i=0;
    if(strstr(buffer,"stem expansion"))i=1;
    if(strstr(buffer,"fuzzy matching"))i=1;
    if(strstr(buffer,"soundex expansion"))i=1;
    if(strstr(buffer,"Number of hits"))i=1;
    if(strstr(buffer,"Synonym expansion"))i=1;
    if(strstr(buffer,"Stem expansion"))i=1;
    if(strstr(buffer,"Stem and case"))i=1;
    if(strstr(buffer,"time for"))i=1;
    if(strstr(buffer,"QUERY"))i=1;
    if(i==1)
    {
      if(strlen(buffer)>100) strcpy(buffer+100,"...");
      strcat(buffer,"<BR>\n");
      str_append(&info,buffer);
    }
  }
  save_info(glp->query_ID,info.string);
  /*printf("%s",info.string); */
  fclose(info_file);
}



