rem
rem
Rem  Copyright (c) 1995 by Oracle Corporation
Rem    NAME
Rem      owautil.sql - package of various OWA utility procedures
Rem    DESCRIPTION
Rem      This file contains one package:
Rem         owa_util    - Utitility procedures/functions for use
Rem                       with the Oracle Web Agent
Rem
Rem    NOTES
Rem      The Oracle Web Agent is needed to use these facilities.
Rem      The package owa is needed to use these facilities.
Rem      The packages htp and htf are needed to use these facilities.
Rem
Rem    MODIFIED   (MM/DD/YY)
Rem     mbookman   08/08/95 -  tablePrint now re-sizes narrow empty tables
Rem     mbookman   07/09/95 -  Creation

REM Creating OWA_UTIL package body...
create or replace package body OWA_UTIL is

   table_border char(1);

   /* The broken line below is intentional */
   NL_CHAR  constant  char(1) := ' ';

     /******************************************************************/
    /* Procedure to link back to the PL/SQL source for your procedure */
   /******************************************************************/
   procedure showsource(cname in varchar2) is
      procname  varchar2(30);
      procowner varchar2(30);

      part1      varchar2(30);
      part2      varchar2(30);
      part1_type integer;
      dblink varchar2(128);
      object_id  integer;

      cursor c1 (n in varchar2, o in varchar2) is
        select line, text from all_source
           where name  = upper(n)
             and owner = upper(o)
           order by type, line;

      line1s integer := 0;
   begin

      dbms_utility.name_resolve(cname, 1, procowner, part1, part2, dblink,
                                part1_type, object_id);
      if part1_type in (7,8)
      then
         procname := part2;
      else
         procname := part1;
      end if;

      htp.header(1,'Source code for '||procname);
      htp.preOpen;
      for source_rec in c1 (procname, procowner)
      loop
         if (source_rec.line = 1)
         then
            line1s := line1s + 1;
            if (line1s = 2)
               then htp.print;
            end if;
         end if;

         htp.prints(translate(source_rec.text,NL_CHAR,' '));
      end loop;
      htp.preClose;
      signature;
   end;

     /**************************************************/
    /* Procedures for printing out an OWA "signature" */
   /**************************************************/
   procedure signature is
   begin
      htp.line;
      htp.p('This page was produced by the ');
      htp.p(htf.bold('Oracle Web Agent')||' on '||
            to_char(sysdate,'Month DD, YYYY HH12:MI PM')||htf.nl);
   end;
 
   procedure signature(cname in varchar2) is
   begin
      signature;
      htp.anchor(owa_util.get_owa_service_path||
                 'owa_util.showsource?cname='||cname,
                 'View PL/SQL source code');
   end;

      /******************************************************/
     /* Procedure for printing a page generated by htp/htf */
    /* in SQL*Plus or SQL*DBA                             */ 
   /******************************************************/
   procedure showpage is
      dbms_buf_size integer;

      buffer   varchar2(510);    /* size = 255 * 2 */
      i        integer;

      sp_loc   integer;
      nl_loc   integer;
   begin
      /* First figure out how large to make the dbms_output buffer */
      dbms_buf_size := (htp.rows_in - htp.rows_out)*255;
      if (dbms_buf_size > 1000000)
      then
         dbms_output.enable(1000000);
      else
         dbms_output.enable(dbms_buf_size);
      end if;
 
      /* Now, loop through, adding lines from htbuf, but   */
      /* never getting larger than 510 characters.         */
      /* If a newline is found, print everything and clear */
      /* the buffer, otherwise, break on the last space    */
      /* possible.  If the last space is past 255 chars,   */
      /* or there is no space at all, then break on 255.   */
      buffer := NULL;
      i := htp.rows_out+1;
      while (i <= htp.rows_in)
      loop
         if (nvl(length(buffer),0) <= 255)
         then
            buffer := buffer||htp.htbuf(i);
            i := i+1;
         end if;

         nl_loc := instr(buffer, NL_CHAR, -1);
         if (nl_loc = 0)
         then
            sp_loc := instr(buffer, ' ', -1);
            if (sp_loc = 0)
            then
               dbms_output.put_line(substr(buffer,1,255));
               buffer := substr(buffer,256);
            else
               if (sp_loc <= 255)
               then
                  dbms_output.put_line(substr(buffer,1,sp_loc-1));
                  buffer := substr(buffer,sp_loc+1);
               else  /* sp_loc > 255 */
                  dbms_output.put_line(substr(buffer,1,255));
                  buffer := substr(buffer,256);
               end if;
            end if;
         else
            /* Always strip out the newlines */
            /* PUT_LINE will put them in.    */
            if (nl_loc <= 255)
            then
               dbms_output.put_line(substr(buffer,1,nl_loc-1));
               buffer := NULL;
            else /* nl_loc > 255 */
               dbms_output.put_line(substr(buffer, 1, 255));
               dbms_output.put_line(substr(buffer, 256, (nl_loc-1) - 256));
               buffer := NULL;
            end if;
         end if;
      end loop;

      htp.rows_out := htp.rows_in;
   end;

     /**************************************************************/
    /* Procedure/function for accessing CGI environment variables */
   /**************************************************************/
   function get_cgi_env(param_name in varchar2) return varchar2 is
      i number := 1;
   begin
      loop
         if (owa.cgi_var_name(i) = upper(param_name))
           then return owa.cgi_var_val(i);
         end if;
 
         i := i+1;
      end loop;
 
   exception
 
      when NO_DATA_FOUND then return NULL;
   end;
 
   procedure print_cgi_env is
      i number := 1;
   begin
      loop
         htp.print(owa.cgi_var_name(i)||' = '||owa.cgi_var_val(i)||htf.nl);
         i := i+1;
      end loop;
 
   exception
 
      when NO_DATA_FOUND then NULL;
   end;

   function get_owa_service_path return varchar2 is
      script_name varchar2(2000);
   begin
      script_name := get_cgi_env('SCRIPT_NAME');

      if (substr(script_name,length(script_name)-1) = '/')
      then
         return script_name;
      else
         return script_name||'/';
      end if;
   end;

   procedure mime_header(ccontent_type in varchar2 DEFAULT 'text/html')
      is
   begin
      htp.prn('Content-type: '||ccontent_type||NL_CHAR||NL_CHAR);
   end;

     /**********************************************/
    /* A couple of handy routines used internally */
   /**********************************************/
   function get_next_col(col_list in  varchar2,
                         loc_in   in  integer,
                         loc_out  out integer) return varchar2
    is
      return_str  varchar2(2000);
      tmp_loc_out integer;
   begin
      tmp_loc_out := instr(col_list, ',', loc_in);
   
      if (tmp_loc_out = 0)
      then
         return_str := substr(col_list, loc_in);
      else
         return_str := substr(col_list, loc_in, tmp_loc_out-loc_in);
      end if;
   
      loc_out := tmp_loc_out;
      return (ltrim(rtrim(return_str)));
   end;

   procedure comma_to_ident_arr(list    in varchar2,
                                arr    out ident_arr,
                                arrlen out integer) is
      token       varchar2(2000);
      tok_counter integer;
      tok_loc_in  integer;
      tok_loc_out integer;
   begin
      if list is null
      then
         arrlen := 0;
      else
         tok_counter := 0;
         tok_loc_in := 1;
         loop
            token := get_next_col(list, tok_loc_in, tok_loc_out);
            tok_counter := tok_counter+1;
            arr(tok_counter) := substr(token,1,30);
            tok_loc_in := tok_loc_out+1;

            if tok_loc_out = 0
            then
               exit;
            end if;
         end loop;
         arrlen := tok_counter;
      end if;
   end;

   function align(cdata        in     varchar2,
                  ncolumn_size in     integer,
                  calign       in     varchar2 DEFAULT 'LEFT') return varchar2
    is
      lalign     integer;
      align_type char(1);
   begin
      align_type := upper(substr(calign,1,1));
      if (align_type = 'L')
      then
         lalign := 1 + nvl(length(cdata),0);
      else
         if (align_type = 'R')
         then
            lalign := ncolumn_size+1;
         else /* align_type = 'C' */
            lalign := 1 + ceil((ncolumn_size - nvl(length(cdata),0))/2)
                        + nvl(length(cdata),0);
         end if;
      end if;

      return (rpad(lpad(nvl(cdata,' '), lalign), ncolumn_size+2)||table_border);
   end;

     /******************************************************************/
    /* Procedures and functions for building HTML and non-HTML tables */
   /******************************************************************/
   /* This is just a function prototype */
   procedure resolve_synonym(csynonym in varchar2,
                             cschema  in varchar2,
                             resolved_name    out varchar2,
                             resolved_owner   out varchar2,
                             resolved_db_link out varchar2);

   procedure resolve_table(cobject in varchar2,
                           cschema in varchar2,
                           resolved_name    out varchar2,
                           resolved_owner   out varchar2,
                           resolved_db_link out varchar2) is
      a varchar2(30);
      b varchar2(30);
      c varchar2(30);
      dblink varchar2(30);
      next_pos binary_integer;
   
      item_name varchar2(30);
      item_owner varchar2(30); 
   
      dummy char(1);
   begin
      dbms_utility.name_tokenize(cobject,a,b,c,dblink,next_pos);
   
      if (c is not null)
      then
         /* For a table, we should see AT MOST owner.table */
         /* If c has a value, we've got owner.table.column */
         /* or owner.package.procedure                     */
         raise_application_error(-20000,'Value '||cobject||' passed to resolve_table is invalid');
      end if;
   
      if (b is not null) 
      then              
         item_owner:=a;
         item_name:=b;
      else
         item_owner:=cschema;
         item_name:=a;
      end if;
   
      select null into dummy from all_tables
         where table_name = upper(item_name)
           and owner      = upper(item_owner);
   
      resolved_name    := item_name;
      resolved_owner   := item_owner;
      resolved_db_link := dblink;
   
   exception
   
      when NO_DATA_FOUND then
      begin
         select null into dummy from all_views
            where view_name = upper(item_name)
              and owner     = upper(item_owner);
   
         resolved_name    := item_name;
         resolved_owner   := item_owner;
         resolved_db_link := dblink;
      exception
   
         when NO_DATA_FOUND then
            resolve_synonym(item_name, item_owner,
                            resolved_name, resolved_owner, resolved_db_link);
   
      end;
   end;

   procedure resolve_synonym(csynonym in varchar2,
                             cschema  in varchar2,
                             resolved_name    out varchar2,
                             resolved_owner   out varchar2,
                             resolved_db_link out varchar2) is
      name    varchar2(30);
      owner   varchar2(30);
      db_link varchar2(128);
      is_also_syn number;
   begin
      select table_name, table_owner, db_link
         into name, owner, db_link
         from all_synonyms
         where synonym_name = upper(csynonym)
           and        owner = upper(cschema);

      if (db_link is null)
      then
         resolve_table(name, owner,
                       resolved_name, resolved_owner, resolved_db_link);
      else
         raise_application_error(-20002,'Cannot resolve remote object'||csynonym);
/*
         declare
            crsr integer;
            ignore integer;
         begin
            crsr := dbms_sql.open_cursor;
            dbms_sql.parse(crsr,
                           'begin y@x(5); '||
                             'owa_util.resolve_table(:cobject, '||
                                                    ':cschema, '||
                                                    ':resolved_name, '||
                                                    ':resolved_owner, '||
                                                    ':resolved_db_link)@'||
                                                    db_link||'; '||
                           'end;', dbms_sql.v7);
            dbms_sql.bind_variable(crsr, ':cobject', name);
            dbms_sql.bind_variable(crsr, ':cschema', owner);
            dbms_sql.bind_variable(crsr, ':resolved_name', name, 30);
            dbms_sql.bind_variable(crsr, ':resolved_owner', owner, 30);
            dbms_sql.bind_variable(crsr, ':resolved_db_link', db_link, 128);
            ignore := dbms_sql.execute(crsr);

            dbms_sql.variable_value(crsr, ':resolved_name', name);
            resolved_name := name;
            dbms_sql.close_cursor(crsr);
         end;
*/
      end if;

   exception
   
      when no_data_found then
      begin
         select table_name, table_owner, db_link
            into name, owner, db_link
            from all_synonyms
            where synonym_name = upper(csynonym)
              and        owner = 'PUBLIC';

         if (db_link is null)
         then resolve_table(name, owner,
                            resolved_name, resolved_owner, resolved_db_link);
         else
            raise_application_error(-20002,'Cannot resolve remote object'||csynonym);
         end if;

      exception
         when no_data_found
         then
            raise_application_error(-20001,'Cannot resolve object'||csynonym);
      end;
   end;
   
   /* DESCRIBE_COLS returns the column_names and datatypes as */
   /* arrays for passing to calc_col_sizes                    */
   procedure describe_cols(
                           ctable       in varchar2,
                           ccolumns     in varchar2,
                           col_names   out ident_arr,
                           col_dtypes  out ident_arr,
                           nnum_cols   out integer
                          )
    is
     col_cursor    integer;
     col_name      varchar2(30);
     col_dtype     varchar2(9);
     col_counter   integer;
     new_row       boolean;
     col_num       integer;
   
     col_loc_in    integer;
     col_loc_out   integer;
     next_col      varchar2(30);
     col_decode    varchar2(2000);
     col_in_clause varchar2(2000);
   
     table_resolved varchar2(30);
     owner_resolved varchar2(30);
     db_link_resolved varchar2(30);
   
     ignore     integer;
   begin
      /* There's no dynamic describe unfortunately. */
      /* We will need to parse out the owner, etc. */
      resolve_table(ctable,USER, 
                    table_resolved,owner_resolved,db_link_resolved);
      
      col_cursor := dbms_sql.open_cursor;
   
      if (ccolumns = '*')
      then
         dbms_sql.parse(col_cursor,
                        'select column_name, data_type '||
                           'from all_tab_columns '||
                           'where table_name = '''||upper(table_resolved)||''' '||
                           '  and owner = '''||upper(owner_resolved)||''' '||
                           'order by column_id',
                        dbms_sql.v7);

         dbms_sql.define_column(col_cursor, 1, col_name, 30);
         dbms_sql.define_column(col_cursor, 2, col_dtype, 9);

         ignore := dbms_sql.execute(col_cursor);

         col_counter := 0;
         loop
            if dbms_sql.fetch_rows(col_cursor) > 0
            then
               dbms_sql.column_value(col_cursor, 2, col_dtype);
               dbms_sql.column_value(col_cursor, 1, col_name);

               col_counter := col_counter + 1;
               col_dtypes(col_counter) := col_dtype;
               col_names(col_counter) := col_name;
            else
               exit;
            end if;
         end loop;
      else
         col_decode := '';
         col_in_clause := '';
         col_loc_in := 1;
         next_col := upper(get_next_col(ccolumns,col_loc_in,col_loc_out));
         col_counter := 1;
   
         while (col_loc_out != 0)
         loop
            col_names(col_counter) := next_col;

            col_decode := col_decode||','''||next_col||''','||col_counter;
            col_in_clause := col_in_clause||''''||next_col||''',';
            col_loc_in := col_loc_out+1;
            next_col := upper(get_next_col(ccolumns,col_loc_in,col_loc_out));
            col_counter := col_counter+1;
         end loop;
   
         col_names(col_counter) := next_col;

         col_decode := col_decode||','''||next_col||''','||col_counter;
         col_in_clause := col_in_clause||''''||next_col||'''';
   
         dbms_sql.parse(col_cursor,
                       'select column_name, data_type, '||
                           'decode(column_name'||col_decode||') '||
                           'from all_tab_columns '||
                           'where table_name = '''||upper(table_resolved)||''' '||
                           '  and owner = '''||upper(owner_resolved)||''' '||
                           '  and column_name in ('||col_in_clause||') '||
                           'order by 3',
                        dbms_sql.v7);

         dbms_sql.define_column(col_cursor, 1, col_name, 30);
         dbms_sql.define_column(col_cursor, 2, col_dtype, 9);
         dbms_sql.define_column(col_cursor, 3, col_num, 9);

         ignore := dbms_sql.execute(col_cursor);

         new_row := dbms_sql.fetch_rows(col_cursor) > 0;
         for i in 1..col_counter
         loop
            if (new_row) AND (i = col_num)
            then
               dbms_sql.column_value(col_cursor, 2, col_dtype);
               col_dtypes(i) := col_dtype;
               new_row := dbms_sql.fetch_rows(col_cursor) > 0;
            else
               col_dtypes(i) := 'VARCHAR2';
            end if;
         end loop;
      end if;
                
      nnum_cols := col_counter;

      dbms_sql.close_cursor(col_cursor);
   end;
 
   procedure eliminate_longs(col_names   in out ident_arr,
                             col_aliases in out ident_arr,
                             col_dtypes  in out ident_arr,
                             num_cols    in out integer,
                             num_aliases in out integer) is
   begin

      for i in 1..num_cols
      loop
         if (col_dtypes(i) = 'LONG') OR (col_dtypes(i) = 'LONG RAW')
         then
            num_cols := num_cols - 1;

            if (i < num_aliases)
            then
               num_aliases := num_aliases - 1;
            end if;

            for j in i..num_cols
            loop
               col_dtypes(j) := col_dtypes(j+1);
               col_names(j) := col_names(j+1);

               if (j <= num_aliases)
               then
                  col_aliases(j) := col_aliases(j+1);
               end if;
            end loop;

            /* To be totally clean, let's null the last values */
            col_names(num_cols+1) := NULL;
            col_dtypes(num_cols+1) := NULL;
            col_aliases(num_aliases+1) := NULL;

            /* Since there is only one LONG allowed in a table, exit */
            exit;
         end if;
      end loop;

   end;

   /* CALC_COL_SIZES will calculate the necessary column sizes   */
   /* for a table.  If an ntable_type = HTML_TABLE, then it      */
   /* merely builds an array of NULLs, one entry for each column */
   /* This is necessary for calls to print_headings.             */
   /* For PRE_TABLEs, CALC_COL_SIZES must scan the table up to   */
   /* the nrow_max-th to determine the widest values.  If        */
   /* nrow_max is NULL, then the entire table is scanned.        */
   procedure calc_col_sizes(ctable      in     varchar2,
                            ntable_type in     integer,
                            ccolumns    in     varchar2,
                            col_names   in     ident_arr,
                            col_dtypes  in     ident_arr,
                            nnum_cols   in     integer,
                            col_aliases in     ident_arr,
                            num_aliases in     integer DEFAULT 0,
                            cclauses    in     varchar2 DEFAULT NULL,
                            nrow_min    in     integer DEFAULT NULL,
                            nrow_max    in     integer DEFAULT NULL,
                            col_sizes   in out num_arr,
                            table_empty    out boolean) is
     crsr     integer;
     ignore   integer;
 
     col_counter integer;
 
     vc_var     varchar2(2000);
     number_var number;
     date_var   date;
     long_var   varchar2(32767);
     raw_var    raw(255);
 
     col_size integer;
 
     row_count number;
   begin
      if ntable_type = HTML_TABLE
      then
         for i in 1..nnum_cols
         loop
            col_sizes(i) := NULL;
         end loop;
      else
         crsr := dbms_sql.open_cursor;
         dbms_sql.parse(crsr,
                        'select '||ccolumns||' from '||ctable||' '||cclauses,
                        dbms_sql.v7);

         for col_counter in 1..nnum_cols
         loop
            if (col_counter <= num_aliases)
            then
               col_sizes(col_counter):= nvl(length(col_aliases(col_counter)),0);
            else
               col_sizes(col_counter):= nvl(length(col_names(col_counter)),0);
            end if;
            if ( (col_dtypes(col_counter) = 'VARCHAR2') OR
                 (col_dtypes(col_counter) = 'CHAR') )
            then
               dbms_sql.define_column(crsr, col_counter, vc_var, 2000);
            else if (col_dtypes(col_counter) = 'NUMBER')
                 then
                    dbms_sql.define_column(crsr,
                                           col_counter, number_var);
                 else if (col_dtypes(col_counter) = 'DATE')
                      then
                         dbms_sql.define_column(crsr,
                                                col_counter, date_var);
                      else if (col_dtypes(col_counter) = 'LONG')
                           then
                              dbms_sql.define_column(crsr, col_counter,
                                                     long_var, 32767);
                           else if (col_dtypes(col_counter) = 'RAW')
                                then
                                   dbms_sql.define_column_raw(crsr, col_counter,
                                                              raw_var, 255);
                                end if;
                           end if;
                      end if;
                 end if;
            end if;
         end loop;

         ignore := dbms_sql.execute(crsr);

         row_count := 0;
         if (nrow_min is NOT NULL)
         then
            while (row_count < nrow_min - 1)
            loop
               if (dbms_sql.fetch_rows(crsr) > 0)
                  then row_count := row_count+1;
                  else exit;
               end if;
            end loop;
         end if;

         while (nrow_max is NULL) or (row_count < nrow_max)
         loop

            if dbms_sql.fetch_rows(crsr) > 0
            then
               row_count := row_count+1;

               for col_counter in 1..nnum_cols
               loop
                  if ( (col_dtypes(col_counter) = 'VARCHAR2')
                    OR (col_dtypes(col_counter) = 'CHAR') )
                  then
                     dbms_sql.column_value(crsr, col_counter, vc_var);
                     col_size := nvl(length(vc_var),0);
                  else if (col_dtypes(col_counter) = 'NUMBER')
                       then
                          dbms_sql.column_value(crsr, col_counter,
                                                number_var);
                          col_size := nvl(length(number_var),0);
                       else if (col_dtypes(col_counter) = 'DATE')
                            then
                               dbms_sql.column_value(crsr, col_counter,
                                                     date_var);
                               col_size := nvl(length(date_var),0);
                            else if (col_dtypes(col_counter) = 'LONG')
                                 then
                                    dbms_sql.column_value(crsr, col_counter,
                                                          long_var);
                                    col_size := nvl(length(long_var),0);
                                 else if (col_dtypes(col_counter) = 'RAW')
                                      then
                                         dbms_sql.column_value_raw(crsr,
                                                          col_counter,
                                                          raw_var);
                                         col_size := nvl(length(raw_var),0);
                                      else
                                         col_size := length('Not Printable');
                                      end if;
                                 end if;
                            end if;
                       end if;
                  end if;

                  if (col_size > col_sizes(col_counter))
                  then
                     col_sizes(col_counter) := col_size;
                  end if;
   
               end loop;

            else
               if row_count = 0
               then
                  table_empty := true;
               else
                  table_empty := false;
               end if;
               exit;
            end if;
         end loop;

         dbms_sql.close_cursor(crsr);
      end if; 
   end;

   /* PRINT_HEADINGS will print the column headings for a table. */
   /* If ccol_aliases is populated, it will use them, else it    */
   /* will use ccol_names.                                       */
   procedure print_headings(
                            ccol_aliases in     ident_arr,
                            num_aliases  in     integer,
                            ccol_names   in     ident_arr,
                            ccol_sizes   in     num_arr,
                            nnum_cols    in     integer,
                            ntable_width in out integer,
                            ntable_type  in     integer
                           ) is
     row_string varchar2(32000);
   begin
      tableHeaderRowOpen(row_string, ntable_width, ntable_type);

      for i in 1..nnum_cols
      loop
         if (i <= num_aliases)
         then
            tableHeader(ccol_aliases(i), ccol_sizes(i), 'CENTER',
                        row_string, ntable_width, ntable_type);
         else
            tableHeader(ccol_names(i), ccol_sizes(i), 'CENTER',
                        row_string, ntable_width, ntable_type);
         end if;
      end loop;

      tableHeaderRowClose(row_string, ntable_width, ntable_type);
   end;

   /* PRINT_ROWS will print the requested rows (nrow_min, nrow_max, */
   /* cclauses) and columns (ccolumns) from the table (ctable)      */
   /* in the specified format (ntable_type).                        */
   /* DESCRIBE_COLS (or a functional equivalent) must be called     */
   /* before calling PRINT_ROWS to populate col_dtypes, col_sizes.  */
   /* PRINT_ROWS returns TRUE if there are more rows (beyond        */
   /* nrow_max) to print.  False otherwise.                         */
   function print_rows(
                       ctable       in varchar2,
                       ntable_type  in integer DEFAULT HTML_TABLE,
                       ccolumns     in varchar2 DEFAULT '*',
                       cclauses     in varchar2 DEFAULT NULL,
                       col_dtypes   in ident_arr,
                       col_sizes    in num_arr,
                       nnum_cols    in integer,
                       ntable_width in integer,
                       nrow_min     in integer DEFAULT 0,
                       nrow_max     in integer DEFAULT NULL
                      ) return boolean
    is
     table_cursor integer;
  
     col_counter integer;
  
     vc_var     varchar2(2000);
     number_var number;
     date_var   date;
     long_var   varchar2(32767);
     raw_var    raw(255);
  
     ignore     integer;
  
     row_string varchar2(32000);

     row_count number;
     more_rows boolean := TRUE;
   begin
      table_cursor := dbms_sql.open_cursor;
      dbms_sql.parse(table_cursor,
                     'select '||ccolumns||' from '||ctable||' '||cclauses,
                     dbms_sql.v7);

      for col_counter in 1..nnum_cols
      loop
         if ( (col_dtypes(col_counter) = 'VARCHAR2') OR 
              (col_dtypes(col_counter) = 'CHAR') )
         then
            dbms_sql.define_column(table_cursor, col_counter, vc_var, 2000);
         else if (col_dtypes(col_counter) = 'NUMBER')
              then
                 dbms_sql.define_column(table_cursor,
                                        col_counter, number_var);
              else if (col_dtypes(col_counter) = 'DATE')
                   then
                      dbms_sql.define_column(table_cursor,
                                             col_counter, date_var);
                   else if (col_dtypes(col_counter) = 'LONG')
                        then
                           dbms_sql.define_column(table_cursor,
                                                  col_counter, long_var, 32767);
                        else if (col_dtypes(col_counter) = 'RAW')
                             then
                                dbms_sql.define_column_raw(table_cursor,
                                                           col_counter,
                                                           raw_var, 32767);
                             end if;
                        end if;
                   end if;
              end if;
         end if;
      end loop;

      ignore := dbms_sql.execute(table_cursor);
   
      row_count := 0;
      if (nrow_min is NOT NULL)
      then
         while (row_count < nrow_min - 1)
         loop
            if (dbms_sql.fetch_rows(table_cursor) > 0)
               then row_count := row_count+1;
               else exit;
            end if;
         end loop;
      end if;

      while (nrow_max is NULL) or (row_count < nrow_max)
      loop
   
         if dbms_sql.fetch_rows(table_cursor) > 0
         then
            row_count := row_count+1;
   
            tableRowOpen(row_string, ntable_type);
   
            for col_counter in 1..nnum_cols
            loop
               if ( (col_dtypes(col_counter) = 'VARCHAR2')
                 OR (col_dtypes(col_counter) = 'CHAR') )
               then
                  dbms_sql.column_value(table_cursor, col_counter, vc_var);
                  tableData(vc_var, col_sizes(col_counter), 'LEFT',
                            row_string, ntable_type);
               else if (col_dtypes(col_counter) = 'NUMBER')
                    then
                       dbms_sql.column_value(table_cursor, col_counter,
                                             number_var);
                       tableData(number_var, col_sizes(col_counter), 'LEFT',
                                 row_string, ntable_type);
                    else if (col_dtypes(col_counter) = 'DATE')
                         then
                            dbms_sql.column_value(table_cursor, col_counter,
                                                  date_var);
                            tableData(date_var, col_sizes(col_counter), 'LEFT',
                                      row_string, ntable_type);
                         else if (col_dtypes(col_counter) = 'LONG')
                              then
                                 dbms_sql.column_value(table_cursor,
                                                       col_counter,
                                                       long_var);
                                 tableData(long_var, col_sizes(col_counter),
                                           'LEFT', row_string, ntable_type);
                              else if (col_dtypes(col_counter) = 'RAW')
                                   then
                                      dbms_sql.column_value_raw(table_cursor,
                                                                col_counter,
                                                                raw_var);
                                      tableData(raw_var, col_sizes(col_counter),
                                               'LEFT', row_string, ntable_type);
                                   else
                                      tableData('Not Printable',
                                                col_sizes(col_counter),
                                               'LEFT', row_string, ntable_type);
                                   end if;
                              end if;
                         end if;
                    end if;
               end if;
               
            end loop;
      
            tableRowClose(row_string, ntable_type); 
         else
            more_rows := FALSE;
            exit;
         end if;
      end loop;

      if (row_count < nrow_min)
      then
         tableRowOpen(row_string, ntable_type);
         tableNoData('LEFT', row_string, nnum_cols, ntable_width, ntable_type);
         tableRowClose(row_string, ntable_type); 
      else
         if (more_rows)
            then more_rows := dbms_sql.fetch_rows(table_cursor) > 0;
         end if;
      end if;

      dbms_sql.close_cursor(table_cursor);

      return more_rows;
   end;

   procedure show_query_columns(ctable in varchar2) is
      table_resolved varchar2(30);
      owner_resolved varchar2(30);
      db_link_resolved varchar2(30);

      cursor cols is
        select column_name from all_tab_columns
          where table_name = upper(table_resolved)
            and owner      = upper(owner_resolved);
   begin
      /* There's no dynamic describe unfortunately. */
      /* We will need to parse out the owner, etc. */
      resolve_table(ctable,USER,
                    table_resolved,owner_resolved,db_link_resolved);

      htp.formHidden('ctable', ctable);
      htp.formHidden('COLS', 'DUMMY');
      for crec in cols loop
         htp.formCheckbox('COLS',crec.column_name);
         htp.print(crec.column_name);
         htp.nl;
      end loop;
      htp.formSubmit(NULL,'Execute Query');
   end;

   function tablePrint(ctable       in varchar2,
                       cattributes  in varchar2 DEFAULT NULL,
                       ntable_type  in integer  DEFAULT HTML_TABLE,
                       ccolumns     in varchar2 DEFAULT '*',
                       cclauses     in varchar2 DEFAULT NULL,
                       ccol_aliases in varchar2 DEFAULT NULL,
                       nrow_min     in number DEFAULT 0,
                       nrow_max     in number DEFAULT NULL) return boolean
    is
     col_names   ident_arr;
     col_aliases ident_arr;
     num_aliases integer;
     col_dtypes  ident_arr;
     col_sizes   num_arr;

     nnum_cols    integer;
     ntable_width integer;

     no_data_len integer;
     inc_len     integer;
     amt_left    integer;

     table_empty boolean;
     more_rows   boolean;
   begin
      describe_cols(ctable, ccolumns, col_names, col_dtypes, nnum_cols);

      if (ccol_aliases is not null)
         then comma_to_ident_arr(ccol_aliases, col_aliases, num_aliases);
         else num_aliases := 0;
      end if;

      calc_col_sizes(ctable, ntable_type, ccolumns, col_names, col_dtypes,
                     nnum_cols, col_aliases, num_aliases, cclauses,
                     nrow_min, nrow_max, col_sizes, table_empty);

      if (table_empty)
      then
         ntable_width := 1;
         for i in 1..nnum_cols
         loop
            ntable_width := ntable_width + col_sizes(i) + 3;
         end loop;

         no_data_len := length('  No Data Found  ');

         if (ntable_width < no_data_len)
         then
            amt_left := no_data_len - ntable_width;
            inc_len := ceil(amt_left/nnum_cols);

            for i in 1..nnum_cols
            loop
               if amt_left > inc_len 
               then
                  col_sizes(i) := col_sizes(i) + inc_len;
                  amt_left := amt_left - inc_len;
               else
                  col_sizes(i) := col_sizes(i) + amt_left;
                  amt_left := 0;
               end if;
            end loop;
         end if;
      end if;

      tableOpen(cattributes, ntable_type);
   
      print_headings(col_aliases, num_aliases, col_names, col_sizes,
                     nnum_cols, ntable_width, ntable_type);

      more_rows := print_rows(ctable, ntable_type, ccolumns, cclauses,
                              col_dtypes, col_sizes, nnum_cols, ntable_width,
                              nrow_min, nrow_max);

      tableClose(ntable_width, ntable_type); 
   
      return(more_rows);
   end;

   procedure tableOpen(cattributes in varchar2 DEFAULT NULL,
                       ntable_type in integer DEFAULT HTML_TABLE)
    is
   begin
      if (ntable_type = HTML_TABLE)
      then
         htp.tableOpen(NULL, NULL, NULL, NULL, cattributes);
      else
         if (cattributes is not null)
         then
            table_border := '|';
         else
            table_border := ' ';
         end if;
         htp.print('<PRE>');
      end if;
   end;
 
   procedure tableCaption(ccaption    in varchar2,
                          calign      in varchar2 DEFAULT 'CENTER',
                          ntable_type in integer  DEFAULT HTML_TABLE) is
   begin
      if (ntable_type = HTML_TABLE)
      then
         htp.tableCaption(ccaption, calign);
      else
         htp.print(ccaption);
      end if;
   end;

   procedure tableHeaderRowOpen(crowstring  in out varchar2,
                                ntable_type in     integer DEFAULT HTML_TABLE)
    is
   begin
      if (ntable_type = HTML_TABLE)
      then
         htp.tableRowOpen;
      else
         crowstring := table_border;
      end if;
   end;
 
   procedure tableHeaderRowOpen(crowstring   in out varchar2,
                                ntable_width    out integer,
                                ntable_type  in     integer DEFAULT HTML_TABLE)
    is
   begin
      if (ntable_type = HTML_TABLE)
      then
         htp.tableRowOpen;
      else
         ntable_width := 1;
         crowstring := table_border;
      end if;
   end;
 
   procedure tableHeader(ccolumn_name in     varchar2,
                         ncolumn_size in     integer,
                         calign       in     varchar2 DEFAULT 'CENTER',
                         crowstring   in out varchar2,
                         ntable_type  in     integer DEFAULT HTML_TABLE)
    is
   begin
      if (ntable_type = HTML_TABLE)
      then
         htp.tableHeader(ccolumn_name);
      else
         crowstring := crowstring||align(ccolumn_name,ncolumn_size,calign);
      end if;
   end;

   procedure tableHeader(ccolumn_name in     varchar2,
                         ncolumn_size in     integer,
                         calign       in     varchar2 DEFAULT 'CENTER',
                         crowstring   in out varchar2,
                         ntable_width in out integer,
                         ntable_type  in     integer DEFAULT HTML_TABLE)
    is
   begin
      if (ntable_type = HTML_TABLE)
      then
         htp.tableHeader(ccolumn_name);
      else
         ntable_width := ntable_width+ncolumn_size+3;
         crowstring := crowstring||align(ccolumn_name,ncolumn_size,calign);
      end if;
   end;

   procedure tableHeaderRowClose(crowstring  in out varchar2,
                                 ntable_type in     integer DEFAULT HTML_TABLE)
    is
   begin
      if (ntable_type = HTML_TABLE)
      then
         htp.tableRowClose;
      else
         htp.print(crowstring);
      end if;
   end;

   procedure tableHeaderRowClose(crowstring   in out varchar2,
                                 ntable_width in     integer,
                                 ntable_type  in     integer DEFAULT HTML_TABLE)
    is
   begin
      if (ntable_type = HTML_TABLE)
      then
         htp.tableRowClose;
      else
         if (table_border = '|')
         then
            htp.print(rpad('-',ntable_width,'-'));
            htp.print(crowstring);
            htp.print(rpad('-',ntable_width,'-'));
         else
            htp.print(' ');
            htp.print(crowstring);
            htp.print(' ');
         end if;
      end if;
   end;

   procedure tableRowOpen(crowstring  in out varchar2,
                          ntable_type in     integer DEFAULT HTML_TABLE)
    is
   begin
      if (ntable_type = HTML_TABLE)
      then
         htp.tableRowOpen;
      else
         crowstring := table_border;
      end if;
   end;

   procedure tableData(cdata        in     varchar2,
                       ncolumn_size in     integer,
                       calign       in     varchar2 DEFAULT 'LEFT',
                       crowstring   in out varchar2,
                       ntable_type  in     integer DEFAULT HTML_TABLE)
    is
   begin
      if (ntable_type = HTML_TABLE)
      then
         htp.tableData(cdata, calign);
      else
         crowstring := crowstring||align(translate(cdata,NL_CHAR,' '),
                                                   ncolumn_size, calign);
      end if;
   end;
 
   procedure tableNoData(calign       in     varchar2 DEFAULT 'LEFT',
                         crowstring   in out varchar2,
                         nnum_cols    in     integer,
                         ntable_width in     integer,
                         ntable_type  in     integer DEFAULT HTML_TABLE)
    is
   begin
      if (ntable_type = HTML_TABLE)
      then
         htp.tableData('No data found', ccolspan=>nnum_cols);
      else
         crowstring := crowstring||align('No data found',ntable_width-4,calign);
      end if;
   end;
 
   procedure tableRowClose(crowstring  in out varchar2,
                           ntable_type in     integer DEFAULT HTML_TABLE)
    is
   begin
      if (ntable_type = HTML_TABLE)
      then
         htp.tableRowClose;
      else
         htp.print(crowstring);
      end if;
   end;

   procedure tableClose(ntable_type in     integer DEFAULT HTML_TABLE)
    is
   begin
      if (ntable_type = HTML_TABLE)
      then
         htp.tableClose;
      else
         htp.print('</PRE>');
      end if;
   end;

   procedure tableClose(ntable_width in integer,
                        ntable_type  in integer DEFAULT HTML_TABLE)
    is
   begin
      if (ntable_type = HTML_TABLE)
      then
         htp.tableClose;
      else
         if (table_border = '|')
         then
            htp.print(rpad('-',ntable_width,'-'));
         else
            htp.print(' ');
         end if;

         htp.print('</PRE>');
      end if;
   end;

end;
/
show errors
