rem
rem
Rem  Copyright (c) 1995 by Oracle Corporation
Rem    NAME
Rem      owautil.sql - package of various OWA utility procedures
Rem    DESCRIPTION
Rem      This file contains one package:
Rem         owa_util    - Utitility procedures/functions for use
Rem                       with the Oracle Web Agent
Rem
Rem    NOTES
Rem      The Oracle Web Agent is needed to use these facilities.
Rem      The package owa is needed to use these facilities.
Rem      The packages htp and htf are needed to use these facilities.
Rem
Rem    MODIFIED   (MM/DD/YY)
Rem     mbookman   07/09/95 -  Creation

REM Creating OWA_UTIL package...
create or replace package OWA_UTIL is

   type ident_arr is table of varchar2(30) index by binary_integer;
   type num_arr is table of number index by binary_integer;

     /******************************************************************/
    /* Procedure to link back to the PL/SQL source for your procedure */
   /******************************************************************/
    /* SHOWSOURCE can take as an argument a procedure, function,     */
    /*   package, package.procedure or package.function.             */
    /* SHOWSOURCE prints the source for the specified stored PL/SQL. */
    /* If package.procedure or package.function are passed, it will  */
    /* print the entire package.                                     */
   procedure showsource(cname in varchar2);

     /**************************************************/
    /* Procedures for printing out an OWA "signature" */
   /**************************************************/
    /* SIGNATURE prints an HTML line followed by a line like:               */
    /*   This page was produced by the Oracle Web Agent on 09/07/95 09:39   */
   procedure signature;

    /* SIGNATURE (cname) prints an HTML line followed by 2 lines like:      */
    /*   This page was produced by the Oracle Web Agent on August 9, 1995
         9:39 AM                                                            */
    /*   View PL/SQL source code (hypertext-link)                           */
    /* SIGNATURE can take as an argument a procedure, function, or package, */
    /*   but not package.procedure or package.function.  See SHOWSOURCE.    */
   procedure signature(cname in varchar2);

      /******************************************************/
     /* Procedure for printing a page generated by htp/htf */
    /* in SQL*Plus or SQL*DBA                             */
   /******************************************************/
    /* SHOWPAGE can be called to print out the results, */
    /* in SQL*Plus or SQL*DBA, of an htp/htf generated  */
    /* page. This is done using dbms_output, and thus   */
    /* is limited to 255 characters per line and an     */
    /* overall buffer size of 1,000,000 bytes.          */
   procedure showpage;

     /**************************************************************/
    /* Procedure/function for accessing CGI environment variables */
   /**************************************************************/
   /* GET_CGI_ENV will return the value of the requested CGI     */
   /* environment variable, or NULL if that value is not set.    */
   function  get_cgi_env(param_name in varchar2) return varchar2;

   /* PRINT_CGI_ENV will print all of the CGI environment        */
   /* variables which OWA has made available to PL/SQL.          */
   procedure print_cgi_env;


/* ***************************************/
/* Jiangang Guo's stuff                  */
/* ***************************************/

--   function  get_cgi_env_wow(param_name in varchar2) return varchar2;
--   procedure print_cgi_env_wow;


   /* MIME_HEADER will output "Content-type: <ccontent_type>\n\n" */
   /* This allows changing the default MIME header which the Web  */
   /* Agent returns.  This MUST be come before any htp.print or   */
   /* htp.prn calls in order to signal the Web Agent not to use   */
   /* the default.                                                */
   procedure mime_header(ccontent_type in varchar2 DEFAULT 'text/html');

   /* GET_OWA_SERVICE_PATH returns the name of the currently           */
   /* with its full virtual path, plus the currently active service    */
   /*  For example, a call to get_owa_service_path could return:       */
   /*     /ows-bin/owa/myservice/                                      */
   function get_owa_service_path return varchar2;

     /******************************************************************/
    /* Procedures and functions for building HTML and non-HTML tables */
   /******************************************************************/

   /* TABLE_TYPE constants */
   HTML_TABLE constant integer := 1;
   PRE_TABLE  constant integer := 2;

   procedure show_query_columns(ctable in varchar2);

   /* TABLEPRINT will print out an entire Oracle table either as   */
   /* an HTML table, or as a "pre-formatted" table.  The table     */
   /* alignment follows the HTML 3.0 current standards for default */
   /* alignment - column headings are CENTERED while table data is */
   /* LEFT justified.                                              */
   /*                                                              */
   /* TABLEPRINT takes the following parameters:                   */
   /*                                                              */
   /* ctable       - the table, view, or synonym name              */
   /* cattributes  - allows you to pass any of the attributes that */
   /*                can be passed to the HTML <TABLE> tag.        */
   /* ntable_type  - HTML_TABLE or PRE_TABLE                       */
   /* ccolumns     - a comma-delimited list of columns from ctable */
   /* cclauses     - any SQL "where" or "order by clauses",        */
   /*  for example : "where deptno = 10"                           */
   /*                "where deptno = 10 order by ename"            */
   /*                "order by deptno"                             */
   /* ccol_aliases - a comma-delimited list of column headings     */
   /* nrow_min     - the first row, of those fetched, to print     */
   /* nrow_max     - the last row, of those fetched, to print      */
   /*                                                              */
   /* Note that RAW COLUMNS are supported, however LONG RAW        */
   /*  are not.  References to LONG RAW columns will print the     */
   /*  result 'Not Printable'.                                     */
   function tablePrint(ctable       in varchar2,
                       cattributes  in varchar2 DEFAULT NULL,
                       ntable_type  in integer  DEFAULT HTML_TABLE,
                       ccolumns     in varchar2 DEFAULT '*',
                       cclauses     in varchar2 DEFAULT NULL,
                       ccol_aliases in varchar2 DEFAULT NULL,
                       nrow_min     in number DEFAULT 0,
                       nrow_max     in number DEFAULT NULL) return boolean;

   /* Lower-level routines for printing out the table */
   procedure comma_to_ident_arr(list    in varchar2,
                                arr    out ident_arr,
                                arrlen out integer);

   procedure tableOpen(cattributes in varchar2 DEFAULT NULL,
                       ntable_type in integer DEFAULT HTML_TABLE);

   procedure tableCaption(ccaption    in varchar2,
                          calign      in varchar2 DEFAULT 'CENTER',
                          ntable_type in integer  DEFAULT HTML_TABLE);

   procedure tableHeaderRowOpen(crowstring  in out varchar2,
                                ntable_type in     integer DEFAULT HTML_TABLE);

   procedure tableHeaderRowOpen(crowstring   in out varchar2,
                                ntable_width    out integer,
                                ntable_type  in     integer DEFAULT HTML_TABLE);

   procedure tableHeader(ccolumn_name in     varchar2,
                         ncolumn_size in     integer,
                         calign       in     varchar2 DEFAULT 'CENTER',
                         crowstring   in out varchar2,
                         ntable_type  in     integer DEFAULT HTML_TABLE);

   procedure tableHeader(ccolumn_name in     varchar2,
                         ncolumn_size in     integer,
                         calign       in     varchar2 DEFAULT 'CENTER',
                         crowstring   in out varchar2,
                         ntable_width in out integer,
                         ntable_type  in     integer DEFAULT HTML_TABLE);

   procedure tableHeaderRowClose(crowstring  in out varchar2,
                                 ntable_type in     integer DEFAULT HTML_TABLE);

   procedure tableHeaderRowClose(crowstring   in out varchar2,
                                 ntable_width in     integer,
                                 ntable_type  in     integer DEFAULT HTML_TABLE);

   procedure tableRowOpen(crowstring  in out varchar2,
                          ntable_type in     integer DEFAULT HTML_TABLE);

   procedure tableData(cdata        in     varchar2,
                       ncolumn_size in     integer,
                       calign       in     varchar2 DEFAULT 'LEFT',
                       crowstring   in out varchar2,
                       ntable_type  in     integer DEFAULT HTML_TABLE);

   procedure tableNoData(calign       in     varchar2 DEFAULT 'LEFT',
                         crowstring   in out varchar2,
                         nnum_cols    in     integer,
                         ntable_width in     integer,
                         ntable_type  in     integer DEFAULT HTML_TABLE);

   procedure tableRowClose(crowstring  in out varchar2,
                           ntable_type in     integer DEFAULT HTML_TABLE);

   procedure tableClose(ntable_type in     integer DEFAULT HTML_TABLE);

   procedure tableClose(ntable_width in integer,
                        ntable_type  in integer DEFAULT HTML_TABLE);

end;
/
show errors

